'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var define = require('./define-18cb4958.js');
require('graphql');

function nodeDefinitions(idFetcher) {
    const nodeInterface = define.Gql.InterfaceType({
        name: 'Node',
        description: 'An object with an ID',
        fields: () => [
            define.Gql.AbstractField({
                name: 'id',
                type: define.Gql.NonNull(define.Gql.ID),
                description: 'The id of the object.',
            }),
        ],
    });
    const nodeField = define.Gql.Field({
        name: 'node',
        type: nodeInterface,
        args: {
            id: define.Gql.Arg({
                type: define.Gql.NonNullInput(define.Gql.ID),
                description: 'The ID of an object',
            }),
        },
        resolve: (_, { id }, context, info) => idFetcher(id, context, info),
    });
    return { nodeInterface, nodeField };
}
const forwardConnectionArgs = {
    after: define.Gql.Arg({ type: define.Gql.String }),
    first: define.Gql.Arg({ type: define.Gql.Int }),
};
const backwardConnectionArgs = {
    before: define.Gql.Arg({ type: define.Gql.String }),
    last: define.Gql.Arg({ type: define.Gql.Int }),
};
const connectionArgs = {
    ...forwardConnectionArgs,
    ...backwardConnectionArgs,
};
const pageInfoType = define.Gql.Object({
    name: 'PageInfo',
    description: 'Information about pagination in a connection.',
    fields: () => [
        define.Gql.Field({
            name: 'hasNextPage',
            type: define.Gql.NonNull(define.Gql.Boolean),
            description: 'When paginating forwards, are there more items?',
        }),
        define.Gql.Field({
            name: 'hasPreviousPage',
            type: define.Gql.NonNull(define.Gql.Boolean),
            description: 'When paginating backwards, are there more items?',
        }),
        define.Gql.Field({
            name: 'startCursor',
            type: define.Gql.String,
            description: 'When paginating backwards, the cursor to continue.',
        }),
        define.Gql.Field({
            name: 'endCursor',
            type: define.Gql.String,
            description: 'When paginating forwards, the cursor to continue.',
        }),
    ],
});
/**
 * Returns Objects for a connection with the given name,
 * and whose nodes are of the specified type.
 */
function connectionDefinitions(config) {
    const { nodeType } = config;
    const name = config.name || nodeType.name;
    // TODO
    const edgeFields = config.edgeFields || (() => []);
    const connectionFields = config.connectionFields || (() => []);
    const edgeType = define.Gql.Object({
        name: name + 'Edge',
        description: 'An edge in a connection.',
        fields: () => [
            define.Gql.Field({
                name: 'node',
                type: nodeType,
                description: 'The item at the end of the edge',
            }),
            define.Gql.Field({
                name: 'cursor',
                type: define.Gql.NonNull(define.Gql.String),
                description: 'A cursor for use in pagination',
            }),
            ...edgeFields(),
        ],
    });
    const connectionType = define.Gql.Object({
        name: name + 'Connection',
        description: 'A connection to a list of items.',
        fields: () => [
            define.Gql.Field({
                name: 'pageInfo',
                type: define.Gql.NonNull(pageInfoType),
                description: 'Information to aid in pagination.',
            }),
            define.Gql.Field({
                name: 'edges',
                type: define.Gql.List(edgeType),
                description: 'A list of edges.',
            }),
            ...connectionFields(),
        ],
    });
    return { edgeType, connectionType };
}

exports.backwardConnectionArgs = backwardConnectionArgs;
exports.connectionArgs = connectionArgs;
exports.connectionDefinitions = connectionDefinitions;
exports.forwardConnectionArgs = forwardConnectionArgs;
exports.nodeDefinitions = nodeDefinitions;
