import { GraphQLInt, GraphQLFloat, GraphQLBoolean, GraphQLID, GraphQLString } from 'graphql';

function builtInScalar(builtInType) {
    return {
        kind: 'Scalar',
        builtInType,
    };
}
var Gql;
(function (Gql) {
    Gql.String = builtInScalar(GraphQLString);
    Gql.Int = builtInScalar(GraphQLInt);
    Gql.Float = builtInScalar(GraphQLFloat);
    Gql.Boolean = builtInScalar(GraphQLBoolean);
    Gql.ID = builtInScalar(GraphQLID);
    function Scalar({ name, description, serialize, parseValue, parseLiteral, }) {
        return {
            kind: 'Scalar',
            graphqlTypeConfig: {
                name,
                description,
                serialize: serialize,
                parseLiteral,
                parseValue,
            },
        };
    }
    Gql.Scalar = Scalar;
    function Enum({ name, description, values, }) {
        return {
            kind: 'Enum',
            name,
            description,
            values,
        };
    }
    Gql.Enum = Enum;
    function Arg({ type, description, default: defaultArg, }) {
        return {
            kind: 'Argument',
            type: type,
            description,
            default: defaultArg,
        };
    }
    Gql.Arg = Arg;
    function DefaultArg(type, defaultArg, description) {
        return {
            kind: 'DefaultArgument',
            type: type,
            description,
            default: defaultArg,
        };
    }
    Gql.DefaultArg = DefaultArg;
    function Field({ name, type, resolve, args, ...options }) {
        return {
            kind: 'Field',
            name,
            type,
            args: args !== null && args !== void 0 ? args : {},
            // if no resolver is defined we fallback to the default GraphQL resolver which is (src) => src[fieldName]
            resolve: typeof resolve === 'function' ? resolve : undefined,
            ...options,
        };
    }
    Gql.Field = Field;
    function AbstractField(opts) {
        return {
            kind: 'AbstractField',
            name: opts.name,
            description: opts.description,
            deprecationReason: opts.deprecationReason,
            args: opts.args,
            type: opts.type,
        };
    }
    Gql.AbstractField = AbstractField;
    function Object({ name, description, interfaces = [], fields, isTypeOf, extensions, }) {
        const obj = {
            kind: 'ObjectType',
            name,
            description,
            interfaces,
            fieldsFn: undefined,
            isTypeOf: isTypeOf,
            extensions,
        };
        obj.fieldsFn = () => fields(obj);
        return obj;
    }
    Gql.Object = Object;
    function InputObject({ name, description, fields, }) {
        let inputObj = {
            kind: 'InputObject',
            name,
            description,
            fieldsFn: null,
        };
        inputObj.fieldsFn = () => fields(inputObj);
        return inputObj;
    }
    Gql.InputObject = InputObject;
    function Union({ name, description, types, resolveType, }) {
        return {
            kind: 'Union',
            name,
            description,
            types,
            resolveType,
        };
    }
    Gql.Union = Union;
    function InterfaceType({ name, description, interfaces = [], fields, }) {
        const obj = {
            kind: 'Interface',
            name,
            description,
            interfaces,
            fieldsFn: undefined,
        };
        obj.fieldsFn = () => fields(obj);
        return obj;
    }
    Gql.InterfaceType = InterfaceType;
    function List(ofType) {
        return {
            kind: 'List',
            ofType: ofType,
        };
    }
    Gql.List = List;
    function ListInput(ofType) {
        return {
            kind: 'ListInput',
            ofType: ofType,
        };
    }
    Gql.ListInput = ListInput;
    function NonNull(ofType) {
        return {
            kind: 'NonNull',
            ofType: ofType,
        };
    }
    Gql.NonNull = NonNull;
    function NonNullInput(ofType) {
        return {
            kind: 'NonNullInput',
            ofType: ofType,
        };
    }
    Gql.NonNullInput = NonNullInput;
    function Query({ name = 'Query', fields, }) {
        return {
            kind: 'ObjectType',
            name,
            interfaces: [],
            fieldsFn: fields,
        };
    }
    Gql.Query = Query;
    function Mutation({ name = 'Mutation', fields, }) {
        return {
            kind: 'ObjectType',
            name,
            interfaces: [],
            fieldsFn: fields,
        };
    }
    Gql.Mutation = Mutation;
    function SubscriptionField({ name, type, args = {}, subscribe, description, deprecationReason, }) {
        return {
            kind: 'SubscriptionField',
            name,
            type,
            args,
            subscribe: subscribe,
            resolve: (value) => value,
            description,
            deprecationReason,
        };
    }
    Gql.SubscriptionField = SubscriptionField;
    function Subscription({ name = 'Subscription', fields, }) {
        return {
            kind: 'SubscriptionObject',
            name,
            fields,
        };
    }
    Gql.Subscription = Subscription;
})(Gql || (Gql = {}));

export { Gql as G };
