import * as graphql from 'graphql';
import type { ScalarType, EnumType, EnumValue, ObjectType, InputObjectType, InterfaceType, UnionType, InputType, OutputType, ArgMap, TOfArgMap, Field, AbstractField, DefaultArgument, Argument, InputFieldMap, SubscriptionField, SubscriptionObjectType, PromiseOrValue, GqlContext } from './types.js';
type ExtensionsMap = {
    field?: {
        [key: string]: any;
    };
    objectType?: {
        [key: string]: any;
    };
};
type ResolvePartialMandatory<Src, Arg, Out> = {
    resolve: (src: Src, args: TOfArgMap<ArgMap<Arg>>, ctx: GqlContext, info: graphql.GraphQLResolveInfo) => PromiseOrValue<Out>;
};
type ResolvePartialOptional<Src, Arg, Out> = {
    resolve?: (src: Src, args: TOfArgMap<ArgMap<Arg>>, ctx: GqlContext, info: graphql.GraphQLResolveInfo) => PromiseOrValue<Out>;
};
export declare namespace Gql {
    const String: ScalarType<string | null | undefined>;
    const Int: ScalarType<number | null | undefined>;
    const Float: ScalarType<number | null | undefined>;
    const Boolean: ScalarType<boolean | null | undefined>;
    const ID: ScalarType<string | null | undefined>;
    function Scalar<Src>({ name, description, serialize, parseValue, parseLiteral, }: {
        name: string;
        description?: string;
        serialize: (src: Src) => any | null;
        parseValue?: (value: unknown) => Src | null;
        parseLiteral?: (value: graphql.ValueNode) => Src | null;
    }): ScalarType<Src | null>;
    function Enum<Src>({ name, description, values, }: {
        name: string;
        description?: string;
        values: Array<EnumValue<Src>>;
    }): EnumType<Src | null>;
    function Arg<Src, TDefault extends Exclude<Src, null | undefined> | undefined>({ type, description, default: defaultArg, }: {
        type: InputType<Src>;
        description?: string;
        default?: TDefault;
    }): Argument<TDefault extends undefined ? Exclude<Src, undefined> : Exclude<Src, null | undefined>>;
    function DefaultArg<Src>(type: InputType<Src>, defaultArg: Exclude<Src, null>, description?: string): DefaultArgument<Exclude<Src, null>>;
    function Field<Key extends string, Src, Out, Arg extends object = {}>({ name, type, resolve, args, ...options }: {
        name: Key;
        type: OutputType<Out>;
        args?: ArgMap<Arg>;
        description?: string;
        deprecationReason?: string;
        extensions?: ExtensionsMap['field'];
    } & (Key extends keyof Src ? Src[Key] extends Out ? ResolvePartialOptional<Src, Arg, Out> : ResolvePartialMandatory<Src, Arg, Out> : ResolvePartialMandatory<Src, Arg, Out>)): Field<Src, any, any>;
    function AbstractField<Out>(opts: {
        name: string;
        type: OutputType<Out>;
        description?: string;
        deprecationReason?: string;
        args?: ArgMap<unknown>;
    }): AbstractField<Out>;
    function Object<Src>({ name, description, interfaces, fields, isTypeOf, extensions, }: {
        name: string;
        description?: string;
        interfaces?: Array<InterfaceType<any>>;
        fields: (self: OutputType<Src | null>) => [Field<Src, any, {}>, ...Field<Src, any, {}>[]];
        isTypeOf?: (src: any, ctx: GqlContext, info: graphql.GraphQLResolveInfo) => boolean;
        extensions?: ExtensionsMap['objectType'] extends undefined ? Record<string, any> : ExtensionsMap['objectType'];
    }): ObjectType<Src | null>;
    function InputObject<Src>({ name, description, fields, }: {
        name: string;
        description?: string;
        fields: (self: InputType<Src | null>) => InputFieldMap<Src>;
    }): InputObjectType<Src | null>;
    function Union<Src>({ name, description, types, resolveType, }: {
        name: string;
        description?: string;
        types: Array<ObjectType<any>> | (() => Array<ObjectType<any>>);
        resolveType: (src: Src) => string;
    }): UnionType<Src | null>;
    function InterfaceType<Src>({ name, description, interfaces, fields, }: {
        name: string;
        description?: string;
        interfaces?: Array<InterfaceType<any>> | (() => Array<InterfaceType<any>>);
        fields: (self: InterfaceType<Src | null>) => Array<AbstractField<any>>;
    }): InterfaceType<Src | null>;
    function List<Src>(ofType: OutputType<Src>): OutputType<Array<Src> | null>;
    function ListInput<Src>(ofType: InputType<Src>): InputType<Array<Src> | null>;
    function NonNull<Src>(ofType: OutputType<Src | null | undefined>): OutputType<Src>;
    function NonNull<Src>(ofType: OutputType<Src | null>): OutputType<Src>;
    function NonNullInput<Src>(ofType: InputType<Src | null | undefined>): InputType<Src>;
    function NonNullInput<Src>(ofType: InputType<Src | null>): InputType<Src>;
    function Query<RootSrc>({ name, fields, }: {
        name?: string;
        fields: () => [Field<RootSrc, any, {}>, ...Field<RootSrc, any, {}>[]];
    }): ObjectType<RootSrc>;
    function Mutation<RootSrc>({ name, fields, }: {
        name?: string;
        fields: () => [Field<RootSrc, any, {}>, ...Field<RootSrc, any, {}>[]];
    }): ObjectType<RootSrc>;
    function SubscriptionField<RootSrc, Out, Arg>({ name, type, args, subscribe, description, deprecationReason, }: {
        name: string;
        type: OutputType<Out>;
        args?: ArgMap<Arg>;
        description?: string;
        deprecationReason?: string;
        subscribe: (src: RootSrc, args: TOfArgMap<ArgMap<Arg>>, ctx: GqlContext, info: graphql.GraphQLResolveInfo) => PromiseOrValue<AsyncIterableIterator<Out>>;
    }): SubscriptionField<RootSrc, Arg, Out>;
    function Subscription<Src>({ name, fields, }: {
        name?: string;
        fields: () => [
            SubscriptionField<Src, any, any>,
            ...SubscriptionField<Src, any, any>[]
        ];
    }): SubscriptionObjectType<Src>;
}
export {};
