import type { Field, InterfaceType, Argument, TOfArgMap, GqlContext, ObjectType } from './types.js';
import { GraphQLResolveInfo } from 'graphql';
export type ConnectionConfig<T> = {
    name?: string;
    nodeType: ObjectType<T | null> | InterfaceType<T | null>;
    edgeFields?: () => Array<Field<Edge<T>, any, any>>;
    connectionFields?: () => Array<Field<Connection<T>, any, any>>;
};
export type Edge<T> = {
    node: T;
    cursor: string;
};
export type PageInfo = {
    hasNextPage: boolean;
    hasPreviousPage: boolean;
    startCursor: string | null;
    endCursor: string | null;
};
export type Connection<T> = {
    edges: Array<Edge<T> | null> | null;
    pageInfo: PageInfo;
};
export type ConnectionArgumentsDefinition = {
    before: Argument<string | null>;
    last: Argument<number | null>;
    after: Argument<string | null>;
    first: Argument<number | null>;
};
export type ConnectionArguments = TOfArgMap<{
    before: Argument<string | null>;
    last: Argument<number | null>;
    after: Argument<string | null>;
    first: Argument<number | null>;
}>;
export type RelayConnectionDefinitions<T> = {
    edgeType: ObjectType<Edge<T> | null>;
    connectionType: ObjectType<Connection<T> | null>;
};
export declare function nodeDefinitions<Src>(idFetcher: (id: string, context: GqlContext, info: GraphQLResolveInfo) => Promise<Src> | Src): {
    nodeInterface: InterfaceType<unknown>;
    nodeField: Field<unknown, any, any>;
};
export declare const forwardConnectionArgs: {
    after: Argument<string | null>;
    first: Argument<number | null>;
};
export declare const backwardConnectionArgs: {
    before: Argument<string | null>;
    last: Argument<number | null>;
};
export declare const connectionArgs: {
    before: Argument<string | null>;
    last: Argument<number | null>;
    after: Argument<string | null>;
    first: Argument<number | null>;
};
/**
 * Returns Objects for a connection with the given name,
 * and whose nodes are of the specified type.
 */
export declare function connectionDefinitions<T>(config: ConnectionConfig<T>): RelayConnectionDefinitions<T>;
